#!/usr/bin/env python3

# /// script
# requires-python = ">=3.9"
# ///

import os.path
import sys


# Get command-line arguments
page = sys.argv[1]

pagename = os.path.dirname(page)
if pagename != "":
    print(f": {pagename}", end="")



"""'linton publish' tests.

Copyright (c) Reuben Thomas 2024.
Released under the GPL version 3, or (at your option) any later version.
"""

from pathlib import Path
from typing import Callable

from pytest import CaptureFixture
from testutils import Case, dir_test, make_tests

from linton import main


test_files_dir = Path(__file__).parent.resolve() / "test-files"

pytestmark = make_tests(
    main,
    test_files_dir,
    Case(
        "from-init",
        ["publish", str(test_files_dir / "publish/from-init/input")],
    ),
)


def test_publish(
    function: Callable[[list[str]], None],
    case: Case,
    fixture_dir: Path,
    capsys: CaptureFixture[str],
    regenerate_expected: bool,
) -> None:
    dir_test(
        function,
        case,
        fixture_dir,
        capsys,
        regenerate_expected,
    )



# © Reuben Thomas <rrt@sc3d.org> 2024-2025
# Released under the GPL version 3, or (at your option) any later version.

import argparse
import importlib.metadata
import logging
import os
import sys
import warnings

from .subcommand import init, publish, serve
from .warnings_util import die, simple_warning


VERSION = importlib.metadata.version("linton")


def main(argv: list[str] = sys.argv[1:]) -> None:
    # Command-line arguments
    parser = argparse.ArgumentParser(
        description="Make a web site from Markdown files and other resources.",
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )
    parser.add_argument(
        "-V",
        "--version",
        action="version",
        version=f"""%(prog)s {VERSION}
© 2024-2025 Reuben Thomas <rrt@sc3d.org>
https://github.com/rrthomas/linton
Distributed under the GNU General Public License version 3, or (at
your option) any later version. There is no warranty.""",
    )
    warnings.showwarning = simple_warning(parser.prog)

    subparsers = parser.add_subparsers(
        required=True, title="subcommands", metavar="SUBCOMMAND"
    )
    publish.add_subparser(subparsers)
    serve.add_subparser(subparsers)
    init.add_subparser(subparsers)

    args = parser.parse_args(argv)
    if "base_url" not in args:
        args.base_url = "/"

    # Expand input
    try:
        args.func(args)
    except Exception as err:
        if "DEBUG" in os.environ:
            logging.error(err, exc_info=True)
        else:
            die(f"{err}")
        sys.exit(1)



#!/usr/bin/env python3

# /// script
# requires-python = ">=3.9"
# ///

import os
import sys
from datetime import datetime


# Get command-line arguments
file = sys.argv[1]

time = os.stat(file).st_mtime
dt = datetime.fromtimestamp(time)
print(dt.strftime("%Y/%m/%d"))



#!/usr/bin/env python3

# /// script
# requires-python = ">=3.9"
# ///

import sys
from typing import Optional


# Read command-line arguments
def maybe_argv(n: int) -> Optional[str]:
    return sys.argv[n] if len(sys.argv) > n else None


text = maybe_argv(1) or "$include(Email.in.txt)"
link_class = maybe_argv(2)
class_text = f' class="{link_class}"' if link_class else ""

print(f'<a{class_text} href="mailto:$include(Email.in.txt)">{text}</a>', end="")



#!/usr/bin/env python3

# /// script
# requires-python = ">=3.9"
# ///

import os.path
import sys
import urllib.parse


# Get command-line arguments
page = sys.argv[1]

# Extract page name and normalize path
path = os.path.dirname(page)
if path == ".":
    path = ""
parents = path.removesuffix("$")

# Generate and print breadcrumb
desc = os.path.basename(parents)
tree = ""
classes = "breadcrumb-item breadcrumb-active"
while parents not in ("", ".", "/"):
    quoted_parents = urllib.parse.quote(parents)
    tree = f'<li class="{classes}">' + \
        f'<a href="$run(path-to-root.in.py,$path)/{quoted_parents}/index.html">{desc}</a>' + \
        f'</li>{tree}'
    classes = "breadcrumb-item"
    parents = os.path.dirname(parents)
    desc = os.path.basename(parents)
print(
    '<li class="breadcrumb-item">' + \
    '<a href="$run(path-to-root.in.py,$path)/index.html">$include(Title.in.txt)</a>' + \
    f'</li>{tree}'
)



"""Linton 'serve' subcommand

© Reuben Thomas <rrt@sc3d.org> 2024-2025
Released under the GPL version 3, or (at your option) any later version.
"""

import argparse
import os
import subprocess
import urllib.parse
from http.server import BaseHTTPRequestHandler, HTTPServer
from pathlib import Path

from xdg import Mime


def run(args: argparse.Namespace) -> None:
    """'serve' command handler"""

    class HTTPRequestHandler(BaseHTTPRequestHandler):
        def do_GET(self) -> None:
            """GET handler"""
            filename = None
            expand = False
            url_path = urllib.parse.unquote(
                urllib.parse.urlparse(self.path.removeprefix(args.base_url)).path
            )
            input_path = Path(args.document_root) / url_path
            if input_path.is_file():
                filename = input_path
            else:
                # If a file is not found but it has a Nancy source file, use it.
                suffix = input_path.suffix
                nancy_suffix = f".nancy{suffix}"
                nancy_source = (
                    input_path.with_suffix(nancy_suffix)
                )
                if os.path.exists(nancy_source):
                    filename = nancy_source
                    url_path = url_path.removesuffix(suffix) + nancy_suffix
                    expand = True
            if filename is None:
                self.send_response(404)
                self.send_header("Content-Type", "text/html")
                self.end_headers()
                self.wfile.write(
                    b"<html><head><title>No such page</title><body>No such page</body></html>"
                )
            else:
                mime_type = str(Mime.get_type2(filename).canonical())
                if expand:
                    output = subprocess.check_output(
                        ["nancy", args.document_root, "-", f"--path={url_path}"],
                    )
                else:
                    with open(filename, "rb") as fh:
                        output = fh.read()
                self.send_response(200)
                self.send_header("Content-Type", mime_type)
                self.end_headers()
                self.wfile.write(output)

    httpd = HTTPServer(("localhost", args.port), HTTPRequestHandler)
    [host, port] = httpd.server_address
    print(f"Connect to server at http://{str(host)}:{port}/index.html")
    httpd.serve_forever()


def add_subparser(subparsers: argparse._SubParsersAction) -> None:
    parser = subparsers.add_parser(
        "serve",
        help="serve a Linton web site locally on your computer, for testing",
    )
    parser.add_argument(
        "--port",
        help="port on which to listen [default: random]",
        type=int,
        default=0,
    )
    parser.add_argument(
        "document_root",
        metavar="DIRECTORY",
        help="directory containing source files [default: current working directory]",
        default=os.getcwd(),
        nargs="?",
    )
    parser.set_defaults(func=run)
